"""Part offer class."""
# Local Modules
# Logistics uses the same offers schema as flagship. However, keeping
# access separate because they may be likely to diverge.
from cofactr.schema.flagship.offer import Offer  # pylint: disable=unused-import

# Was this price originally in a different currency than USD?

# def calculate_tariffs(quant, destination_country) -> float:
#     return  # estimated tar

# def is_exportable(destination_country) -> bool:
#     return
